;DotBlox64d Test
;ATMega8 @ 8 MHz internal R/C

.include "../include/m168def.inc"

.def	column = R15

.def	temp = R16
.def	temp2 = R17
.def	count = R18

.equ	ROW_PORT = PORTD
.equ	COL_PORT = PORTB

;===============================================================================

.dseg
	RAM_Frame: .byte 8 ;one byte per column, MSB = topmost pixel

;===============================================================================

.cseg
.org 0x000
	rjmp	reset
.org OVF0addr
	rjmp	mux
	
;===============================================================================

mux:
	;multiplexing interrupt
	push	temp
	push	ZL
	push	ZH
	in	temp, SREG
	push	temp
	
	;select next display column
	inc	column
	sbrc	column, 3
	clr	column

	;read column data from RAM
	ldi	ZL, LOW(RAM_Frame)
	ldi	ZH, HIGH(RAM_Frame)
	add	ZL, column
	brcc	PC+2
	inc	ZH
	ld	temp, Z
	
	;calculate column bit-mask
	ldi	ZL, 0x01
	sbrc	column, 0
	lsl	ZL
	sbrc	column, 1
	lsl	ZL
	sbrc	column, 1
	lsl	ZL
	sbrc	column, 2
	swap	ZL
	com	ZL
	
	;switch to new column
	ldi	ZH, 0xFF
	out	COL_PORT, ZH
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	out	ROW_PORT, temp
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	rjmp	PC+1
	out	COL_PORT, ZL
	
	pop	temp
	out	SREG, temp
	pop	ZH
	pop	ZL
	pop	temp
	reti
	
;===============================================================================

reset:
	;set stackpointer
	ldi	temp, HIGH(RAMEND)
	out	SPH, temp
	ldi	temp, LOW(RAMEND)
	out	SPL, temp
	
	;init ports
	ldi	temp, 0xFF
	out	DDRB, temp
	out	PORTB, temp
	ldi	temp, 0x00
	out	DDRC, temp
	ldi	temp, 0x3F
	out	PORTC, temp
	ldi	temp, 0xFF
	out	DDRD, temp
	ldi	temp, 0x00
	out	PORTD, temp
	
	;init multiplexing timer
	ldi	temp, 0x02 ;clk/8
	out	TCCR0B, temp
	ldi	temp, 1<<TOIE0
	sts	TIMSK0, temp
	
	;init RAM
	ldi	ZL, LOW(RAM_Frame)
	ldi	ZH, HIGH(RAM_Frame)
	ldi	temp, 0x01
	st	Z+, temp
	ldi	temp, 0x03
	st	Z+, temp
	ldi	temp, 0x07
	st	Z+, temp
	ldi	temp, 0x0F
	st	Z+, temp
	ldi	temp, 0x1F
	st	Z+, temp
	ldi	temp, 0x3F
	st	Z+, temp
	ldi	temp, 0x7F
	st	Z+, temp
	ldi	temp, 0xFF
	st	Z+, temp
	
	;init registers
	clr	column
	
	;enable interrupts
	sei
	
;-------------------------------------------------------------------------------

loop:
	;simple moving animation
	ldi	count, 3
	ldi	temp, 0
	ldi	temp2, 0
delay:
	dec	temp2
	brne	delay
	dec	temp
	brne	delay
	dec	count
	brne	delay
	
	ldi	ZL, LOW(RAM_Frame)
	ldi	ZH, HIGH(RAM_Frame)
	ld	temp2, Z
	com	temp2
	ldi	count, 7
move:
	ldd	temp, Z+1
	st	Z+, temp
	dec	count
	brne	move
	st	Z, temp2

	rjmp	loop
	